require 'rails_helper'

module ApiUserAuth
  RSpec.describe AuthController, type: :controller do
    routes { ApiUserAuth::Engine.routes }

    context 'Create' do
      it 'Valid' do
        expect(AuthUser.count).to eq(0)
        post :create, params: { email: 'user@mail.com', password: '123456' }
        expect(AuthUser.count).to eq(1)

        expect(response).to have_http_status(201)
        expect(resp_json[:email]).to eq('user@mail.com')
        expect(resp_json[:auth_token]).not_to be_blank
        expect(resp_json[:id]).to eq(AuthUser.last.id)
        expect(resp_json[:is_new]).to be_truthy
        # expect(AuthUser.count).to eq(1)
        # post :create, params: { email: 'user@mail.com', password: '123456' }
        # expect(AuthUser.count).to eq(1)
        # expect(response).to have_http_status(200)
        # expect(resp_json[:email]).to eq('user@mail.com')
        # expect(resp_json[:auth_token]).not_to be_blank
      end

      it 'Invalid' do
        expect(AuthUser.count).to eq(0)

        post :create, params: { email: 'user@mail.com' }
        expect(response).to have_http_status(422)
        expect(AuthUser.count).to eq(0)
        expect(resp_json[:message]).not_to be_blank

        post :create, params: { password: '123456' }
        expect(response).to have_http_status(422)
        expect(AuthUser.count).to eq(0)
        expect(resp_json[:message]).not_to be_blank

        post :create, params: { email: 'user@mail.com', password: '' }
        expect(response).to have_http_status(422)
        expect(AuthUser.count).to eq(0)
        expect(resp_json[:message]).not_to be_blank

        post :create, params: { password: '123456', email: '' }
        expect(response).to have_http_status(422)
        expect(AuthUser.count).to eq(0)
        expect(resp_json[:message]).not_to be_blank

        post :create, params: { email: 'user@mail.com', password: '123456' }
        expect(AuthUser.count).to eq(1)
        expect(response).to have_http_status(201)

        post :create, params: { email: 'user@mail.com', password: '123456' }
        expect(AuthUser.count).to eq(1)
        expect(response).to have_http_status(422)
        expect(resp_json[:message]).not_to be_blank
      end
    end

    context 'Login' do
      before do
        AuthUser.create_by_params(email: 'user@mail.com', password: '123456')
      end

      it 'Valid' do
        expect(AuthUser.count).to eq(1)
        expect(AuthUser.last.email).to eq('user@mail.com')

        post :login, params: { email: 'user@mail.com', password: '123456' }
        expect(response).to have_http_status(200)
        expect(resp_json[:email]).to eq('user@mail.com')
        expect(resp_json[:auth_token]).not_to be_blank
        expect(resp_json[:id]).to eq(AuthUser.last.id)
      end

      it 'Invalid' do
        expect(AuthUser.count).to eq(1)
        expect(AuthUser.last.email).to eq('user@mail.com')

        post :login, params: { email: 'user@mail.com' }
        expect(response).to have_http_status(422)
        expect(resp_json[:message]).not_to be_blank

        post :login, params: { password: '123456' }
        expect(response).to have_http_status(422)
        expect(resp_json[:message]).not_to be_blank

        post :login, params: { email: 'user2@mail.com', password: '123456' }
        expect(response).to have_http_status(401)
        expect(resp_json[:message]).not_to be_blank

        post :login, params: { email: 'user@mail.com', password: '123457' }
        expect(response).to have_http_status(401)
        expect(resp_json[:message]).not_to be_blank
      end
    end

    context 'Password' do
      let(:auth_user) do
        AuthUser.create_by_params(email: 'user@mail.com', password: '123456')
      end

      it 'Valid' do
        expect(auth_user.valid_password?('123456')).to be_truthy
        auth_user.update(code: '123456')

        patch :password, params: {
          email: 'user@mail.com', password: '123457', code: '123456'
        }
        expect(response).to have_http_status(200)
        auth_user.reload
        expect(auth_user.valid_password?('123456')).to be_falsey
        expect(auth_user.valid_password?('123457')).to be_truthy
      end

      it 'Invalid' do
        expect(auth_user.valid_password?('123456')).to be_truthy
        auth_user.update(code: '123456')

        patch :password, params: {
          email: 'user1@mail.com', password: '123457', code: '123456'
        }
        expect(response).to have_http_status(422)
        expect(resp_json[:message]).not_to be_blank

        patch :password, params: {
          email: 'user@mail.com', password: '123457', code: '123457'
        }
        expect(response).to have_http_status(422)
        expect(resp_json[:message]).not_to be_blank

        patch :password, params: {
          email: 'user@mail.com', password: '123457'
        }
        expect(response).to have_http_status(422)
        expect(resp_json[:message]).not_to be_blank

        patch :password, params: {
          code: '123457', password: '123457'
        }
        expect(response).to have_http_status(422)
        expect(resp_json[:message]).not_to be_blank
      end
    end

    context 'Provider' do
      context 'Google' do
        it 'Valid' do
          allow(ApiUserAuth::Providers::Google).to(
            receive_message_chain(:get_user).and_return(
              email: 'user@mail.com', name: 'User Name'
            )
          )

          expect(AuthUser.count).to eq(0)
          post :provider, params: { provider: 'google', token: 'token' }
          expect(AuthUser.count).to eq(1)
          expect(response).to have_http_status(200)
          expect(resp_json[:email]).to eq('user@mail.com')
          expect(resp_json[:auth_token]).not_to be_blank
          expect(resp_json[:is_new]).to be_truthy
          expect(resp_json[:id]).to eq(AuthUser.last.id)

          post :provider, params: { provider: 'google', token: 'token' }
          expect(AuthUser.count).to eq(1)
          expect(resp_json[:email]).to eq('user@mail.com')
          expect(resp_json[:auth_token]).not_to be_blank
          expect(resp_json[:is_new]).to be_falsey
          expect(resp_json[:id]).to eq(AuthUser.last.id)
        end

        it 'Invalid' do
          expect(AuthUser.count).to eq(0)
          post :provider, params: { provider: 'google', token: 'token' }
          expect(AuthUser.count).to eq(0)
          expect(response).to have_http_status(422)
          expect(resp_json[:message]).not_to be_blank

          post :provider, params: { provider: 'google2', token: 'token' }
          expect(AuthUser.count).to eq(0)
          expect(response).to have_http_status(422)
          expect(resp_json[:message]).not_to be_blank

          post :provider, params: { provider: 'google' }
          expect(AuthUser.count).to eq(0)
          expect(response).to have_http_status(422)
          expect(resp_json[:message]).not_to be_blank
        end
      end

      context 'Facebook' do
        it 'Valid' do
          allow(ApiUserAuth::Providers::Facebook).to(
            receive_message_chain(:get_user).and_return(
              email: 'user@mail.com', name: 'User Name'
            )
          )
          expect(AuthUser.count).to eq(0)
          post :provider, params: { provider: 'facebook', token: 'token' }
          expect(AuthUser.count).to eq(1)

          expect(response).to have_http_status(200)
          expect(resp_json[:email]).to eq('user@mail.com')
          expect(resp_json[:auth_token]).not_to be_blank
          expect(resp_json[:is_new]).to be_truthy
          expect(resp_json[:id]).to eq(AuthUser.last.id)

          post :provider, params: { provider: 'facebook', token: 'token' }
          expect(AuthUser.count).to eq(1)
          expect(resp_json[:email]).to eq('user@mail.com')
          expect(resp_json[:auth_token]).not_to be_blank
          expect(resp_json[:is_new]).to be_falsey
          expect(resp_json[:id]).to eq(AuthUser.last.id)
        end

        it 'Invalid' do
          expect(AuthUser.count).to eq(0)
          post :provider, params: { provider: 'facebook', token: 'token' }
          expect(AuthUser.count).to eq(0)
          expect(response).to have_http_status(422)
          expect(resp_json[:message]).not_to be_blank

          post :provider, params: { provider: 'facebook2', token: 'token' }
          expect(AuthUser.count).to eq(0)
          expect(response).to have_http_status(422)
          expect(resp_json[:message]).not_to be_blank

          post :provider, params: { provider: 'facebook' }
          expect(AuthUser.count).to eq(0)
          expect(response).to have_http_status(422)
          expect(resp_json[:message]).not_to be_blank
        end
      end

      context 'Instagram' do
        it 'Valid' do
          allow(ApiUserAuth::Providers::Instagram).to(
            receive_message_chain(:get_user).and_return(
              email: 'user@mail.com', name: 'User Name'
            )
          )
          expect(AuthUser.count).to eq(0)
          post :provider, params: { provider: 'instagram', token: 'token' }
          expect(AuthUser.count).to eq(1)

          expect(response).to have_http_status(200)
          expect(resp_json[:email]).to eq('user@mail.com')
          expect(resp_json[:auth_token]).not_to be_blank
          expect(resp_json[:is_new]).to be_truthy
          expect(resp_json[:id]).to eq(AuthUser.last.id)

          post :provider, params: { provider: 'instagram', token: 'token' }
          expect(AuthUser.count).to eq(1)
          expect(resp_json[:email]).to eq('user@mail.com')
          expect(resp_json[:auth_token]).not_to be_blank
          expect(resp_json[:is_new]).to be_falsey
          expect(resp_json[:id]).to eq(AuthUser.last.id)
        end

        it 'Invalid' do
          expect(AuthUser.count).to eq(0)
          post :provider, params: { provider: 'instagram', token: 'token' }
          expect(AuthUser.count).to eq(0)
          expect(response).to have_http_status(422)
          expect(resp_json[:message]).not_to be_blank

          post :provider, params: { provider: 'instagram2', token: 'token' }
          expect(AuthUser.count).to eq(0)
          expect(response).to have_http_status(422)
          expect(resp_json[:message]).not_to be_blank

          post :provider, params: { provider: 'instagram' }
          expect(AuthUser.count).to eq(0)
          expect(response).to have_http_status(422)
          expect(resp_json[:message]).not_to be_blank
        end
      end
    end

    context 'Forgot password' do
      it 'Valid' do
        auth_user = AuthUser.create_by_params(
          email: 'user@mail.com', password: '123456'
        )
        expect(auth_user.valid_password?('123456')).to be_truthy
        expect(auth_user.code).to be_blank

        patch :forgot_password, params: {
          email: 'user@mail.com'
        }
        expect(response).to have_http_status(200)
        auth_user.reload
        expect(auth_user.code).not_to be_blank
      end

      it 'Invalid' do
        auth_user = AuthUser.create_by_params(
          email: 'user@mail.com', password: '123456'
        )
        expect(auth_user.valid_password?('123456')).to be_truthy
        expect(auth_user.code).to be_blank

        patch :forgot_password, params: {
          email: 'user1@mail.com'
        }
        expect(response).to have_http_status(422)
        expect(resp_json[:message]).not_to be_blank
        auth_user.reload
        expect(auth_user.code).to be_blank
      end
    end

    context 'Logout' do
      let(:auth_user) do
        AuthUser.create_by_params(email: 'user@mail.com', password: '123456')
      end

      it 'Valid' do
        expect(auth_user.valid_password?('123456')).to be_truthy
        expect(auth_user.auth_tokens.count).to eq(1)
        token = auth_user.auth_tokens.last

        request.headers['HTTP_AUTHORIZATION'] = "Bearer #{token}"
        delete :logout
        expect(response).to have_http_status(200)
        auth_user.reload
        expect(auth_user.auth_tokens.count).to eq(0)
      end

      it 'Invalid' do
        expect(auth_user.valid_password?('123456')).to be_truthy
        expect(auth_user.auth_tokens.count).to eq(1)
        token = auth_user.auth_tokens.last

        request.headers['HTTP_AUTHORIZATION'] = "Bearer #{SecureRandom.uuid}"
        delete :logout
        expect(response).to have_http_status(400)
        auth_user.reload
        expect(auth_user.auth_tokens.count).to eq(1)
      end

    end

    context 'Add provider' do
      let(:auth_user) do
        AuthUser.create_by_params(email: 'user@mail.com', password: '123456')
      end

      it 'Valid' do
        allow(ApiUserAuth::Providers::Google).to(
          receive_message_chain(:get_user).and_return(
            email: 'user@mail.com', name: 'User Name', provider: 'google',
            id: '102030'
          )
        )

        token = auth_user.auth_tokens.last
        request.headers['HTTP_AUTHORIZATION'] = "Bearer #{token}"
        expect(AuthUser.count).to eq(1)
        expect(auth_user.provider_tokens.count).to eq(0)

        params = {
          provider: 'google', token: 'token'
        }

        post :add_provider, params: params

        expect(auth_user.provider_tokens.count).to eq(1)

        post :provider, params: { provider: 'google', token: 'token' }
        expect(response).to have_http_status(200)
        expect(resp_json[:auth_token]).not_to be_blank
        # expect(resp_json[:google_id]).to eq('102030')
      end

    end
  end
end
